-------------------------------------------------------------------------------
-- File Author: networkerror
-- File Revision: 36
-- File Date: 2009-06-24T07:22:06Z
--
-- Automatically adds the user to the LFG channel on login and when
-- leaving a group, without interfering with actual use of the LFG tool.
--
-- Send suggestions, comments, and bugs to merial.kilrogg@gmail.com.
-------------------------------------------------------------------------------

AlwaysLFG = LibStub("AceAddon-3.0"):NewAddon("AlwaysLFG",
                                             "AceConsole-3.0",
                                             "AceEvent-3.0");

local Version = "2.0.1";

local Options = {
  type = "group",
  get = function(item) return AlwaysLFG.db.profile[item[#item]] end,
  set = function(item, value) AlwaysLFG.db.profile[item[#item]] = value; AlwaysLFG:UpdateLFG() end,
  args = {
    leader = {
      type = "toggle",
      desc = "Set Leader option.",
      name = "Leader",
      width = "half",
      order = 1,
    },
    damage = {
      type = "toggle",
      desc = "Set Damage option.",
      name = "Damage",
      width = "half",
      order = 3,
    },
    tank = {
      type = "toggle",
      desc = "Set Tank option.",
      name = "Tank",
      width = "half",
      disabled = function()
        SetLFGRoles(true,true,true,true)
        local leader, tank, healer, damage = GetLFGRoles()
        if (tank == false) then
          return true
        end
      end,
      order = 3,
    },
    healer = {
      type = "toggle",
      desc = "Set Healer option.",
      name = "Healer",
      width = "half",
      disabled = function() 
        SetLFGRoles(true,true,true,true)
        local leader, tank, healer, damage = GetLFGRoles()
        if (healer == false) then
          return true
        end
      end,
      order = 4,
    },
    comment = {
      type = "input",
      desc = "Sets the comment that AlwaysLFG uses in the LFG tool.",
      name = "LFG Comment",
      width = "double",
      order = 5,
    },
    auto = {
      type = "toggle",
      desc = "Automatically join LFG channel when leaving groups.",
      name = "Join When Leaving Groups",
      width = "double",
      order = 6,
    },
    verbose = {
      type = "toggle",
      desc = "Show console messages when resetting LFG parameters.",
      name = "Verbose",
      width = "double",
      order = 7,
    },
  },
}

local SlashOptions = {
  type = "group",
  handler = AlwaysLFG,
  args = {
    config = {
      type = "execute",
      desc = "Open configuration dialog",
      name = "Configure",
      func = function()
        InterfaceOptionsFrame_OpenToCategory(AlwaysLFG.optionFrames.profiles);
        InterfaceOptionsFrame_OpenToCategory(AlwaysLFG.optionFrames.main);
      end,
    },
    join = {
      type = "execute",
      desc = "Join LFG channel",
      name = "Join LFG",
      func = "JoinLFG",
    },
    leave = {
      type = "execute",
      desc = "Leave LFG channel",
      name = "Leave LFG",
      func = "LeaveLFG",
    },
  },
};

local SlashCmds = {
  "alfg",
  "alwayslfg",
};

local Defaults = {
  profile = {
    leader = false,
    damage = true,
    tank = false,
    healer = false,
    comment = "Listening to LFG channel using AlwaysLFG",
    auto = true,
    verbose = false,
  },
};

-- Check whether we're already in LFG/LFM.
function AlwaysLFG:InLFG()
  local _, _, _, _, _, _, _, _, _, queued, lfgStatus, lfmStatus =
    GetLookingForGroup();
  return (lfgStatus or lfmStatus);
end

-- Update all Blizzard UI elements.
function AlwaysLFG:UpdateLFG()
  AlwaysLFG:GetRole()
  LFGFrame_UpdateRolesChosen();
  LFGFrame_UpdateRoleBoxes();
  SetLFGComment(self.db.profile.comment);
  LFGFrame_Update();
  LFMFrame_Update();
end

function AlwaysLFG:GetRole()
  if (self.db.profile.leader) then
    LFGFrameLeaderCheckButton:SetChecked(true)
    SetLFGRoles(true,true,true,true)
  else
    LFGFrameLeaderCheckButton:SetChecked(false)
  end
  if (self.db.profile.damage) then
    LFGFrameRoleButton1:SetChecked(true);
  else
    LFGFrameRoleButton1:SetChecked(false);
  end
  if (self.db.profile.tank) then
    LFGFrameRoleButton2:SetChecked(true);
  else
    LFGFrameRoleButton2:SetChecked(false);
  end
  if (self.db.profile.healer) then
    LFGFrameRoleButton3:SetChecked(true);
  else
    LFGFrameRoleButton3:SetChecked(false);
  end
end  

   -- Add ourselves to LFM if we're not already LFG/LFM.
function AlwaysLFG:JoinLFG()
  if not self:InLFG() then
    ClearLookingForGroup();
    ClearLookingForMore();

    AlwaysLFG:GetRole()

    SetLookingForGroup(1, 5, 1);
    SetLFGComment(self.db.profile.comment);

    self:UpdateLFG();

    if self.db.profile.verbose then
      self:Print("Modified LFG settings.");
    end
  end
end

-- Leave the LFG channel.
function AlwaysLFG:LeaveLFG()
  ClearLookingForGroup();
  ClearLookingForMore();

  self:UpdateLFG();
end

-- Join LFG the first time we join general chat (i.e., on login).
function AlwaysLFG:CHAT_MSG_CHANNEL_NOTICE(_, event)
  if not self.initialJoin and event == "YOU_JOINED" then
    self:JoinLFG();
    self.initialJoin = true;
  end
end

-- Join LFG whenever we leave a group.
function AlwaysLFG:PARTY_MEMBERS_CHANGED()
  local nowInGroup =
    (GetRealNumPartyMembers() > 0 or GetRealNumRaidMembers() > 0);
  if self.inGroup and not nowInGroup then
    self:JoinLFG();
  end
  self.inGroup = nowInGroup;
end

-- We're alive!
function AlwaysLFG:OnInitialize()
  -- Load our database.
  self.db = LibStub("AceDB-3.0"):New("AlwaysLFGDB", Defaults, "Default");

  -- Set up our config options.
  local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
  
  local config = LibStub("AceConfig-3.0");
  config:RegisterOptionsTable("AlwaysLFG", SlashOptions, SlashCmds);

  local registry = LibStub("AceConfigRegistry-3.0");
  registry:RegisterOptionsTable("AlwaysLFG Options", Options);
  registry:RegisterOptionsTable("AlwaysLFG Profiles", profiles);

  local dialog = LibStub("AceConfigDialog-3.0");
  self.optionFrames = {
    main     = dialog:AddToBlizOptions("AlwaysLFG Options", "AlwaysLFG"),
    profiles = dialog:AddToBlizOptions("AlwaysLFG Profiles", "Profiles",
                                       "AlwaysLFG");
  };

  -- Set initial state.
  self.inGroup = false;
  self.initialJoin = false;

  -- Register for events.
  self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE");
  self:RegisterEvent("PARTY_MEMBERS_CHANGED");

  if self.db.profile.debug then
    self:Print("AlwaysLFG v" .. Version .. " loaded.");
  end
end
